/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XLED software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XLED source distribution.
 *
 * $Id: lamled.c,v 1.2 2002/12/11 19:06:38 brbarret Exp $
 *
 *	Function:	- lam_led*() and lam_rled*() wrappers
 *			- cache # processes and process index
 *			- lam_led*() hide nodeid by using groupd and
 *			  work only for single static LED daemon
 */

#include <stdlib.h>

#include <events.h>
#include <ledreq.h>
#include <net.h>
#include <portable.h>
#include <preq.h>
#include <terror.h>
#include <typical.h>

/*
 * local variables
 */
static int4		lednode = NOTNODEID;	/* LED daemon nodeid */
static int4		nprocs = -1;		/* total # processes */
static int4		procidx = -1;		/* my proces index */

/*
 * local functions
 */
static int4		getlednode();
static int		ledaction();

/*
 *	lam_rledinit
 *
 *	Function:	- initialize process information on given node
 *			- cache the info for later use
 *	Accepts:	- target nodeid
 *			- process index
 *			- total # processes
 *	Returns:	- 0 or ERROR
 */
int
lam_rledinit(node, proc, nproc)

int4			node;
int4			proc;
int4			nproc;

{
	procidx = proc;
	nprocs = nproc;
	return(ledaction(node, LEQINIT, (int4) -1));
}

/*
 *	lam_ledinit
 *
 *	Function:	- initialize process information
 *			- cache the info for later use
 *	Accepts:	- process index
 *			- total # processes
 *	Returns:	- 0 or ERROR
 */
int
lam_ledinit(proc, nproc)

int4			proc;
int4			nproc;

{
	int4		node;
	
	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	procidx = proc;
	nprocs = nproc;
	return(ledaction(node, LEQINIT, (int4) -1));
}

/*
 *	lam_rledclr
 *
 *	Function:	- clear process LEDs on given node
 *	Accepts:	- target nodeid
 *	Returns:	- 0 or ERROR
 */
int
lam_rledclr(node)

int4			node;

{
	return(ledaction(node, LEQOFF, (int4) -1));
}

/*
 *	lam_ledclr
 *
 *	Function:	- clear process LEDs
 *			- locate LED daemon
 *	Returns:	- 0 or ERROR
 */
int
lam_ledclr()

{
	int4		node;

	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	return(lam_rledclr(node));
}

/*
 *	lam_rledoff
 *
 *	Function:	- turn an LED off on given node
 *	Accepts:	- target node
 *			- LED number
 *	Returns:	- 0 or ERROR
 */
int
lam_rledoff(node, lednum)

int4			node;
int			lednum;

{
	if ((lednum < -1) || (lednum >= LEMAXNLEDS)) {
		errno = EINVAL;
		return(LAMERROR);
	}

	return(ledaction(node, LEQOFF, ((int4) 1) << lednum));
}

/*
 *	lam_ledoff
 *
 *	Function:	- turn an LED off
 *			- locate LED daemon
 *	Accepts:	- LED number
 *	Returns:	- 0 or ERROR
 */
int
lam_ledoff(lednum)

int			lednum;

{
	int4		node;

	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	return(lam_rledoff(node, lednum));
}

/*
 *	lam_rledon
 *
 *	Function:	- turn an LED on on given node
 *	Accepts:	- target node
 *			- LED number
 *	Returns:	- 0 or ERROR
 */
int
lam_rledon(node, lednum)

int4			node;
int			lednum;

{
	if ((lednum < -1) || (lednum >= LEMAXNLEDS)) {
		errno = EINVAL;
		return(LAMERROR);
	}

	return(ledaction(node, LEQON, ((int4) 1) << lednum));
}

/*
 *	lam_ledon
 *
 *	Function:	- turn an LED on
 *			- locate LED daemon
 *	Accepts:	- LED number
 *	Returns:	- 0 or ERROR
 */
int
lam_ledon(lednum)

int			lednum;

{
	int4		node;

	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	return(lam_rledon(node, lednum));
}

/*
 *	lam_rledisset
 *
 *	Function:	- check if LED is set on given node
 *	Accepts:	- target node
 *			- LED number
 *	Returns:	- 1 if set, 0 if not set, or ERROR
 */
int
lam_rledisset(node, lednum)

int4			node;
int			lednum;

{
	struct lestate	ledinfo;

	if ((lednum < 0) || (lednum >= LEMAXNLEDS)) {
		errno = EINVAL;
		return(LAMERROR);
	}

	if (procidx < 0) {
		errno = ENOTPROCESS;
		return(LAMERROR);
	}

	if (lam_rledstate(node, procidx, &ledinfo)) return(LAMERROR);

	return((ledinfo.les_mask & (((int4) 1) << lednum)) ? 1 : 0);
}

/*
 *	lam_ledisset
 *
 *	Function:	- check if LED is set
 *			- locate LED daemon
 *	Accepts:	- LED number
 *	Returns:	- 1 if set, 0 if not set, or ERROR
 */
int
lam_ledisset(lednum)

int			lednum;

{
	int4		node;

	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	return(lam_rledisset(node, lednum));
}

/*
 *	lam_rledgetnum
 *
 *	Function:	- get # of LEDs available on given node
 *	Accepts:	- target node
 *	Returns:	- # of LEDs or ERROR
 */
int
lam_rledgetnum(node)

int4			node;

{
	struct lestate	ledinfo;

	if (procidx < 0) {
		errno = ENOTPROCESS;
		return(LAMERROR);
	}

	if (lam_rledstate(node, procidx, &ledinfo)) return(LAMERROR);

	return((int) ledinfo.les_nleds);
}

/*
 *	lam_ledgetnum
 *
 *	Function:	- get # of LEDs available
 *			- locate LED daemon
 *	Returns:	- # of LEDs or ERROR
 */
int
lam_ledgetnum()

{
	int4		node;

	node = getlednode();
	if (node == NOTNODEID) return(LAMERROR);

	return(lam_rledgetnum(node));
}

/*
 *	ledaction
 *
 *	Function:	- take an on/off action on an LED on target node
 *	Accepts:	- target node
 *			- request code
 *			- LED mask
 *	Returns:	- 0 or ERROR
 */
static int
ledaction(nodeid, req, mask)

int4			nodeid;
int4			req;
int4			mask;

{
	if ((procidx < 0) || (nprocs <= 0)) {
		errno = ENOTPROCESS;
		return(LAMERROR);
	}

	return(lam_rledop(nodeid, req, procidx, nprocs, mask));
}

/*
 *	getlednode
 *
 *	Function:	- get/cache LED node if necessary
 *	Returns:	- LED nodeid or NOTNODEID
 */
static int4
getlednode()

{
	struct pstate	*pps;		/* process stat array */
	int		i, j;
	int4		nnodes;		/* # nodes in multicomputer */
	int4		*pnodes;	/* nodes in multicomputer */
	int		nps;		/* ret'n # processes */

	if (lednode != NOTNODEID) {
		return(lednode);
	}

	nnodes = getnall();
	if (nnodes < 0) return(NOTNODEID);

	pnodes = (int4 *) malloc((unsigned) (nnodes * sizeof(int4)));
	if (pnodes == 0) return(NOTNODEID);

	if (getall(pnodes, nnodes)) {
		free((char *) pnodes);
		return(NOTNODEID);
	}
	
	pps = (struct pstate *) malloc((unsigned) PMAX *
			sizeof(struct pstate));

	if (pps == 0) {
		free((char *) pnodes);
		return(NOTNODEID);
	}

	for (i = 0; i < nnodes; ++i) {
		nps = rpstate(pnodes[i], SELECT_ALL, 0, pps, PMAX);
		if (nps < 0) return(NOTNODEID);

		for (j = 0; j < nps; ++j) {

			if (! strcmp("xled", pps[j].ps_name)) {
				lednode = pnodes[i];
				free((char *) pnodes);
				free((char *) pps);
				return(lednode);
			}
		}
	}

	free((char *) pnodes);
	free((char *) pps);
	return(NOTNODEID);
}
